-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- CLK_Generator chip to generate CLK_A_SQ = 10 Hz and CLK_B_SQ = 1 Hz 
-- Strategy: Schematic using frequency dividers and T_FF to square signals

-- This circuit is adapted from the schematic at:
-- https://digsys.upc.edu/csd/P08/img_CLK_generator_strategy.jpg
---------------------------------------------------
-- Project P8  
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
-- OSC_CLK_in = 50 MHz(Intel Terasic DE10-Lite)
-- CLK_A_SQ to be connected to the MUX_4 Ch2 
-- CLK_B_SQ to be connected to the MUX_4 Ch3 
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	
			OSC_CLK_in	:	IN		STD_LOGIC;
			CD 			:	IN		STD_LOGIC;
			CE			:	IN		STD_LOGIC;
			CLK_A_SQ	:	OUT	STD_LOGIC;
			CLK_B_SQ	:	OUT	STD_LOGIC	
		);
END CLK_Generator;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_2M5	IS                 
		PORT(	
			CD,CLK,CE	: 	IN  std_logic;
			TC2500000	:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_10 IS                 
		PORT(
			CD,CLK,CE	: 	IN  std_logic;
			TC10		:	OUT std_logic
		);
	END COMPONENT;	
	
	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_2CLK_A	:	std_logic;
	SIGNAL Pulsed_2CLK_B	:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_2M5
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> CE,
			TC2500000	=> Pulsed_2CLK_A				  
		);

	Chip2	:  Freq_Div_10
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_2CLK_A,
			TC10 	=> Pulsed_2CLK_B					  
		);
	
	Chip3	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2CLK_A,
			Q 		=> CLK_A_SQ				  
		);		

	
	Chip4	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2CLK_B,
			Q 		=> CLK_B_SQ				  
		);	
	
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
