--------------------------------------------------------------------------------
-- An example of a chip design: DeMUX_16 
--------------------------------------------------------------------------------
-- DeMUX_16 using plan B behavioural approach where the truth table is
-- copied to VHDL using "std_match" to be able to use "don't care" terms

-- Adapted from the Fig. 3 schematic in lecture L2.2 
-- https://digsys.upc.edu/csd/P02/L02_2/L2_2.html

---------------------------------------------------
-- Exercise P2  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

ENTITY DeMUX_16 IS
	PORT (	-- Data channels declared as individual wires
		Ch7, Ch6, Ch5, Ch4		: 	OUT	STD_LOGIC;  -- 1 bit per channel
		Ch3, Ch2, Ch1, Ch0		: 	OUT	STD_LOGIC;  -- 1 bit per channel
		Ch8, Ch9, Ch10, Ch11	: 	OUT	STD_LOGIC;  -- 1 bit per channel
		Ch12, Ch13, Ch14, Ch15	: 	OUT	STD_LOGIC;  -- 1 bit per channel			
				
-- S(3..0) to distribute 16 channels declared as a vector
		S						:	IN 	STD_LOGIC_VECTOR (3 downto 0);	
		E_L						: 	IN 	STD_LOGIC;
		D						: 	IN	STD_LOGIC	-- 1 bit input data
	);

END DeMUX_16;

ARCHITECTURE truth_table OF DeMUX_16 IS


-- The wire vectors to interface the truth table artefact
SIGNAL  	X_in 	: STD_LOGIC_VECTOR (5 downto 0);
SIGNAL  	Y_out 	: STD_LOGIC_VECTOR (15 downto 0);
BEGIN
 	 	-- First the disable situation (32 lines)
	Y_out <="0000000000000000" when std_match(X_in,"1-----") else  
		-- Now the sixteen ones in the table 
			"0000000000000001" when std_match(X_in,"000001") else  
			"0000000000000010" when std_match(X_in,"000011") else  
			"0000000000000100" when std_match(X_in,"000101") else  
			"0000000000001000" when std_match(X_in,"000111") else  
			"0000000000010000" when std_match(X_in,"001001") else  
			"0000000000100000" when std_match(X_in,"001011") else  
			"0000000001000000" when std_match(X_in,"001101") else  
			"0000000010000000" when std_match(X_in,"001111") else  		 
			"0000000100000000" when std_match(X_in,"010001") else  
			"0000001000000000" when std_match(X_in,"010011") else  
			"0000010000000000" when std_match(X_in,"010101") else  
			"0000100000000000" when std_match(X_in,"010111") else  
			"0001000000000000" when std_match(X_in,"011001") else  
			"0010000000000000" when std_match(X_in,"011011") else  
			"0100000000000000" when std_match(X_in,"011101") else  
			"1000000000000000" when std_match(X_in,"011111") else 			
		 -- and finally the remaining sixteen zeros 
			"0000000000000000";  
		
-- Output buffers
Ch15 <= Y_out(15); 
Ch14 <= Y_out(14); 
Ch13 <= Y_out(13); 
Ch12 <= Y_out(12); 
Ch11 <= Y_out(11); 
Ch10 <= Y_out(10); 
Ch9  <= Y_out(9); 
Ch8  <= Y_out(8);
Ch7  <= Y_out(7); 
Ch6  <= Y_out(6); 
Ch5  <= Y_out(5); 
Ch4  <= Y_out(4); 
Ch3  <= Y_out(3); 
Ch2  <= Y_out(2); 
Ch1  <= Y_out(1); 
Ch0  <= Y_out(0); 

-- Input buffers
X_in(4 downto 1) <= S;
X_in(5) <= E_L;
X_in(0) <= D;
 	 		 		   	   
END truth_table;