--------------------------------------------------------------------------------
-- Multiplexer of 4 channels: MUX_4
-- Plan A, structural: truth table--> equations
---------------------------------------------------
-- Equation PoS from minilog: 
--  Y= (E_L').(S1+S0+CH0).(S1+S0'+CH1).(S1'+S0+CH2).(S1'+S0'+CH3)

-- CSD project P2 on standard combinational circuits  
-- https://digsys.upc.edu/csd/P02/L02_1/L2_1.html#MUX
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY MUX_4 IS
	PORT (								-- Channel inputs
		Ch0, Ch1, Ch2, Ch3	: 	IN 	STD_LOGIC;  
										-- 2 bits to select 4 channels
		S					:	IN 	STD_LOGIC_VECTOR (1 downto 0);	
		E_L					: 	IN 	STD_LOGIC;
		Y					: 	OUT	STD_LOGIC
	);
END MUX_4;

ARCHITECTURE logic_equations_PoS OF MUX_4 IS  

BEGIN
--  Y= (E_L').(S1+S0+CH0).(S1+S0'+CH1).(S1'+S0+CH2).(S1'+S0'+CH3)
	Y <=	(S(1) or S(0) or Ch0)			and
			(S(1) or not(S(0)) or Ch1	)	and
			(not(S(1)) or S(0) or Ch2)		and
			(not(S(1)) or not(S(0)) or Ch3)	and			
			(not(E_L));
		   	 		 		   	   
END logic_equations_PoS;
