-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- LAB2 example prototype. Top circuit using plan C2 

-- This circuit is adapted from the schematic at:
-- https://digsys.upc.edu/csd/P02/lab02/MUX_DeMUX/img_FPGA_schematic.jpg
---------------------------------------------------
-- Project P2  
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY MUX_DeMUX IS  
	PORT(	
			-- The four channels to multiplex 
		T_Ch0, T_Ch1, T_Ch2, T_Ch3:	IN	STD_LOGIC;  
			-- Enable active-low signal
		TE_L	:	IN	STD_LOGIC;									
		TS		:	IN	STD_LOGIC_VECTOR(1 downto 0);-- Select 4 channels
			-- MUX_4 output output 
		TX		:	OUT	STD_LOGIC;		-- MUX data out
			-- Enable active-low signal
		RE_L	:	IN	STD_LOGIC;							
		RS		:	IN	STD_LOGIC_VECTOR(3 downto 0);-- Distribute 16 channels
			-- DeMUX data input	
		RX		:	IN	STD_LOGIC;	-- DeMUX data input	
			-- Output channels for signal distribution
		R_Ch	:	OUT	STD_LOGIC_VECTOR(15 downto 0)
	);
END MUX_DeMUX;

ARCHITECTURE schematic OF MUX_DeMUX  IS  
-- Components
	COMPONENT MUX_4 IS                 
	PORT (								-- Channel inputs
		Ch0, Ch1, Ch2, Ch3	: 	IN 	STD_LOGIC;  
										-- 2 bits to select 4 channels
		S					:	IN 	STD_LOGIC_VECTOR (1 downto 0);	
		E_L					: 	IN 	STD_LOGIC;
		Y					: 	OUT	STD_LOGIC
	);
	END COMPONENT;	
	
	COMPONENT DeMUX_16 IS                 
	PORT (	-- Data channels declared as individual wires
		Ch7, Ch6, Ch5, Ch4		: 	OUT	STD_LOGIC;  -- 1 bit per channel
		Ch3, Ch2, Ch1, Ch0		: 	OUT	STD_LOGIC;  
		Ch8, Ch9, Ch10, Ch11	: 	OUT	STD_LOGIC;  
		Ch12, Ch13, Ch14, Ch15	: 	OUT	STD_LOGIC;  			
				
-- S(3..0) to distribute 16 channels declared as a vector
		S						:	IN 	STD_LOGIC_VECTOR (3 downto 0);	
		E_L						: 	IN 	STD_LOGIC;
		D						: 	IN	STD_LOGIC	-- 1 bit input data
	);
	END COMPONENT;	

-- Signals for connecting components together 
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	MUX_4
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			E_L			=> TE_L,
			S 			=> TS,
			Ch0			=> T_Ch0,
			Ch1			=> T_Ch1, 				  
			Ch2			=> T_Ch2, 
			Ch3 		=> T_Ch3,  			
			Y			=> TX
		);

	Chip2	: 	DeMUX_16
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			E_L			=> RE_L,
			D			=> RX,			
			S 			=> RS,
			Ch0			=> R_Ch(0),
			Ch1			=> R_Ch(1), 				  
			Ch2			=> R_Ch(2), 
			Ch3 		=> R_Ch(3), 
			Ch4			=> R_Ch(4),
			Ch5			=> R_Ch(5), 				  
			Ch6			=> R_Ch(6), 
			Ch7 		=> R_Ch(7), 
			Ch8			=> R_Ch(8),
			Ch9			=> R_Ch(9), 				  
			Ch10		=> R_Ch(10), 
			Ch11		=> R_Ch(11), 
			Ch12		=> R_Ch(12),
			Ch13		=> R_Ch(13), 				  
			Ch14		=> R_Ch(14), 
			Ch15 		=> R_Ch(15)			
		);

-- Connections and logic between components

END schematic ;
