-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- LAB2 example prototype. Top circuit using plan C2 

-- This circuit is adapted from the schematic at:
-- https://digsys.upc.edu/csd/P02/lab02/lab2.html#Prot
---------------------------------------------------
-- Project P2  
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY MUX_DeMUX_top IS  
	PORT(	
		OSC_CLK_in	:	IN	STD_LOGIC;	-- DE10-Lite board oscillator 
		T_Ch0		:	IN	STD_LOGIC;  -- Two of the four channels 
		T_Ch1		:	IN	STD_LOGIC;	
			-- Enable active - low signals
		TE_L	:	IN	STD_LOGIC;			
		TS		:	IN	STD_LOGIC_VECTOR(1 downto 0);-- Select 4 channels		
		RE_L	:	IN	STD_LOGIC;			
		RS		:	IN	STD_LOGIC_VECTOR(3 downto 0);-- Distribute 16 channels
		R_Ch	:	OUT	STD_LOGIC_VECTOR(15 downto 0);-- Output channels
		TX 		:	OUT	STD_LOGIC;
		RX 		:	IN	STD_LOGIC		
	);
END MUX_DeMUX_top;

ARCHITECTURE schematic_planC2 OF MUX_DeMUX_top  IS  
-- Components
	COMPONENT CLK_Generator IS                 
      PORT(	
			OSC_CLK_in	:	IN	STD_LOGIC;
			CD 			:	IN	STD_LOGIC;
			CE			:	IN	STD_LOGIC;
			CLK_A_SQ	:	OUT	STD_LOGIC;
			CLK_B_SQ	:	OUT	STD_LOGIC	
		);
	END COMPONENT;	
	
	COMPONENT MUX_DeMUX IS                 
	PORT(	
		T_Ch0, T_Ch1, T_Ch2, T_Ch3	:	IN	STD_LOGIC;  
		TE_L	:	IN	STD_LOGIC;
		TS		:	IN	STD_LOGIC_VECTOR(1 downto 0);
		TX		:	OUT	STD_LOGIC;	
		RX		:	IN	STD_LOGIC;			
		RE_L	:	IN	STD_LOGIC;	
		RS		:	IN	STD_LOGIC_VECTOR(3 downto 0);
		R_Ch	:	OUT	STD_LOGIC_VECTOR(15 downto 0)
	);
	END COMPONENT;	

-- Signals for connecting components together 
	SIGNAL T_Ch2, T_Ch3 :	STD_LOGIC;
	
	BEGIN  
-- Instantiation of components
	Chip1	: 	MUX_DeMUX
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			T_Ch0		=> T_Ch0,
			T_Ch1		=> T_Ch1,
			T_Ch2		=> T_Ch2,
			T_Ch3		=> T_Ch3,
			TS 			=> TS,
			TE_L		=> TE_L,
			TX			=> TX,
			
			RX			=> RX,			
			RE_L		=> RE_L,			  
			RS			=> RS,				
			R_Ch		=> R_Ch
		);

	Chip2	: CLK_Generator
		PORT MAP (
	-- from component name 	=> to signal or port name	
			OSC_CLK_in		=> OSC_CLK_in,
			CE 				=> '1',
			CD 				=> '0',
			CLK_A_SQ		=> T_Ch2,
			CLK_B_SQ 		=> T_Ch3				  
		);		

-- Connections and logic between components
	
END schematic_planC2 ;
